@@ -7,7 +7,8 @@ from eventlet.event import Event
 from kombu import Connection, Exchange, Queue
 from kombu.pools import producers
 
-from nameko.exceptions import MethodNotFound, RemoteErrorWrapper
+from nameko.exceptions import (
+    MethodNotFound, RemoteErrorWrapper, UnknownService)
 from nameko.messaging import (
     queue_consumer, HeaderEncoder, HeaderDecoder, AMQP_URI_CONFIG_KEY)
 from nameko.dependencies import (
@@ -294,7 +295,26 @@ class MethodProxy(HeaderEncoder):
 
         msg = {'args': args, 'kwargs': kwargs}
 
-        conn = Connection(container.config[AMQP_URI_CONFIG_KEY])
+        conn = Connection(
+            container.config[AMQP_URI_CONFIG_KEY],
+            transport_options={'confirm_publish': True},
+        )
+
+        # We use the `mandatory` flag in `producer.publish` below to catch rpc
+        # calls to non-existent services, which would otherwise wait forever
+        # for a reply that will never arrive.
+        #
+        # However, the basic.return ("no one is listening for topic") is sent
+        # asynchronously and conditionally, so we can't wait() on the channel
+        # for it (will wait forever on successful delivery).
+        #
+        # Instead, we use (the rabbitmq extension) confirm_publish
+        # (https://www.rabbitmq.com/confirms.html), which _always_ sends a
+        # reply down the channel. Moreover, in the case where no queues are
+        # bound to the exchange (service unknown), the basic.return is sent
+        # first, so by the time kombu returns (after waiting for the confim)
+        # we can reliably check for returned messages.
+
         routing_key = '{}.{}'.format(self.service_name, self.method_name)
 
         exchange = get_rpc_exchange(container)
@@ -314,11 +334,15 @@ class MethodProxy(HeaderEncoder):
                 msg,
                 exchange=exchange,
                 routing_key=routing_key,
+                mandatory=True,
                 reply_to=reply_to_routing_key,
                 headers=headers,
                 correlation_id=correlation_id,
             )
 
+            if not producer.channel.returned_messages.empty():
+                raise UnknownService(self.service_name)
+
         _log.debug('Waiting for RPC reply event %s', self,
                    extra=worker_ctx.extra_for_logging)
         resp_body = reply_event.wait()
